import is from '@sindresorhus/is';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/** buildAggregation takes arguments passed to a query or filter and transforms them into a query clause */

function buildAggregation(args) {
  switch (args.length) {
    case 3:
      return buildAggregation3(args);

    case 4:
      return buildAggregation4(args);

    case 5:
      return buildAggregation5(args);

    default:
      throw new TypeError('invalid arguments');
  }
}

function buildAggregation3(_ref) {
  var _name2, _ref3;

  var name = _ref[0],
      type = _ref[1],
      fieldOrConfig = _ref[2];

  if (is.string(fieldOrConfig)) {
    var _name, _ref2;

    var field = fieldOrConfig;
    return _ref2 = {}, _ref2[name] = (_name = {}, _name[type] = {
      field: field
    }, _name), _ref2;
  }

  var config = fieldOrConfig;
  return _ref3 = {}, _ref3[name] = (_name2 = {}, _name2[type] = _extends({}, config), _name2), _ref3;
}

function buildAggregation4(_ref4) {
  var name = _ref4[0],
      type = _ref4[1],
      fieldOrConfig = _ref4[2],
      configOrBuilder = _ref4[3];

  if (is.string(fieldOrConfig)) {
    var _name3, _ref6;

    var field = fieldOrConfig;

    if (is.function_(configOrBuilder)) {
      var _extends2, _ref5;

      var builder = configOrBuilder;
      return _ref5 = {}, _ref5[name] = _extends((_extends2 = {}, _extends2[type] = {
        field: field
      }, _extends2), builder().build()), _ref5;
    }

    var config = configOrBuilder;
    return _ref6 = {}, _ref6[name] = (_name3 = {}, _name3[type] = _extends({
      field: field
    }, config), _name3), _ref6;
  }

  if (is.function_(configOrBuilder)) {
    var _extends3, _ref7;

    var _config = fieldOrConfig;
    var _builder = configOrBuilder;
    return _ref7 = {}, _ref7[name] = _extends((_extends3 = {}, _extends3[type] = _extends({}, _config), _extends3), _builder().build()), _ref7;
  }

  throw new TypeError('invalid arguments');
}

function buildAggregation5(_ref8) {
  var _extends4, _ref9;

  var name = _ref8[0],
      type = _ref8[1],
      field = _ref8[2],
      config = _ref8[3],
      builder = _ref8[4];
  return _ref9 = {}, _ref9[name] = _extends((_extends4 = {}, _extends4[type] = _extends({
    field: field
  }, config), _extends4), builder().build()), _ref9;
}

/** buildClause takes arguments passed to a query or filter and transforms them into a query clause */

function buildClause(data, args) {
  switch (args.length) {
    case 1:
      return buildClause1(data, args);

    case 2:
      return buildClause2(args);

    case 3:
      return buildClause3(args);

    case 4:
      return buildClause4(args);

    case 5:
      return buildClause5(args);

    default:
      throw new TypeError('invalid args');
  }
}

function buildClause1(data, _ref) {
  var builder = _ref[0];
  var built = builder().build();
  return (data.parent ? built[data.parent] : built.filter || built.query) || {};
}

function buildClause2(_ref2) {
  var _ref4;

  var type = _ref2[0],
      fieldOrConfig = _ref2[1];

  if (is.string(fieldOrConfig)) {
    var _ref3;

    var field = fieldOrConfig;
    return _ref3 = {}, _ref3[type] = {
      field: field
    }, _ref3;
  }

  var config = fieldOrConfig;
  return _ref4 = {}, _ref4[type] = _extends({}, config), _ref4;
}

function buildClause3(_ref5) {
  var type = _ref5[0],
      fieldOrConfig = _ref5[1],
      valueOrConfigOrBuilder = _ref5[2];

  if (is.string(fieldOrConfig)) {
    var _type3, _ref8;

    var field = fieldOrConfig;

    if (is.function_(valueOrConfigOrBuilder)) {
      var _type, _ref6;

      var builder = valueOrConfigOrBuilder;
      return _ref6 = {}, _ref6[type] = (_type = {}, _type[field] = builder().build(), _type), _ref6;
    }

    if (is.plainObject(valueOrConfigOrBuilder)) {
      var _type2, _ref7;

      var config = valueOrConfigOrBuilder;
      return _ref7 = {}, _ref7[type] = (_type2 = {}, _type2[field] = config, _type2), _ref7;
    }

    var value = valueOrConfigOrBuilder;
    return _ref8 = {}, _ref8[type] = (_type3 = {}, _type3[field] = value, _type3), _ref8;
  }

  if (is.function_(valueOrConfigOrBuilder)) {
    var _ref9;

    var _config = fieldOrConfig;
    var _builder = valueOrConfigOrBuilder;
    return _ref9 = {}, _ref9[type] = _extends({}, _config, {}, _builder().build()), _ref9;
  }

  throw new TypeError('invalid argument');
}

function buildClause4(_ref10) {
  var _extends3, _ref12;

  var type = _ref10[0],
      field = _ref10[1],
      value = _ref10[2],
      configOrBuilder = _ref10[3];

  if (is.function_(configOrBuilder)) {
    var _extends2, _ref11;

    var builder = configOrBuilder;
    return _ref11 = {}, _ref11[type] = _extends((_extends2 = {}, _extends2[field] = value, _extends2), builder().build()), _ref11;
  }

  var config = configOrBuilder;
  return _ref12 = {}, _ref12[type] = _extends((_extends3 = {}, _extends3[field] = value, _extends3), config), _ref12;
}

function buildClause5(_ref13) {
  var _extends4, _ref14;

  var type = _ref13[0],
      field = _ref13[1],
      value = _ref13[2],
      config = _ref13[3],
      builder = _ref13[4];
  return _ref14 = {}, _ref14[type] = _extends((_extends4 = {}, _extends4[field] = value, _extends4), config, {}, builder().build()), _ref14;
}

function deduplicateSort(configs) {
  var seenFields = new Set();
  return configs.reverse().filter(function (config) {
    var field = Object.keys(config)[0];

    if (seenFields.has(field) && field !== '_geo_distance') {
      return false;
    }

    seenFields.add(field);
    return true;
  }).reverse();
}
/** buildSort takes arguments passed to the sort function and transforms them into configs */

function buildSort(args) {
  switch (args.length) {
    case 1:
      return buildSort1(args);

    case 2:
      return buildSort2(args);

    default:
      throw new TypeError('invalid arguments');
  }
}

function buildSort1(_ref) {
  var fieldOrConfigArray = _ref[0];

  if (is.string(fieldOrConfigArray)) {
    var _ref2;

    var field = fieldOrConfigArray;
    return _ref2 = {}, _ref2[field] = {
      order: 'asc'
    }, _ref2;
  }

  var configArray = fieldOrConfigArray;
  return configArray.map(function (item) {
    if (is.string(item)) {
      var _ref3;

      var _field = item;
      return _ref3 = {}, _ref3[_field] = {
        order: 'asc'
      }, _ref3;
    } else {
      var keys = Object.keys(item);

      if (keys.length !== 1) {
        throw new TypeError('invalid argument');
      }

      var _field2 = keys[0];
      var orderOrConfig = item[_field2];

      if (is.string(orderOrConfig)) {
        var _ref4;

        var order = orderOrConfig;
        return _ref4 = {}, _ref4[_field2] = {
          order: order
        }, _ref4;
      } else {
        var _ref5;

        var config = orderOrConfig;
        return _ref5 = {}, _ref5[_field2] = _extends({}, config), _ref5;
      }
    }
  });
}

function buildSort2(_ref6) {
  var _ref8;

  var field = _ref6[0],
      orderOrConfig = _ref6[1];

  if (is.string(orderOrConfig)) {
    var _ref7;

    var order = orderOrConfig;
    return _ref7 = {}, _ref7[field] = {
      order: order
    }, _ref7;
  }

  var config = orderOrConfig;
  return _ref8 = {}, _ref8[field] = _extends({}, config), _ref8;
}

function flatMap(items, callbackfn) {
  var results = [];
  var idx = 0;

  for (var _iterator = items, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var item = _ref;
    var result = callbackfn(item, idx++, items);

    if (Array.isArray(result)) {
      for (var _iterator2 = result, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref2 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref2 = _i2.value;
        }

        var r = _ref2;
        results.push(r);
      }
    } else {
      results.push(result);
    }
  }

  return results;
}

/**
 * simplifyClauses transforms an array into a single value if length === 1,
 * otherwise it returns the original array.
 */

function simplifyClauses(items) {
  if (items.length === 1) {
    return items[0];
  }

  return items;
}

function buildBool(data) {
  if (data.or.length === 0 && data.not.length === 0 && !data.filter) {
    if (data.and.length === 1) {
      return data.and[0];
    } else if (data.and.length === 0) {
      return undefined;
    }
  }

  var totalCount = data.and.length + data.or.length + data.not.length;
  return {
    bool: _extends({}, data.and.length ? {
      must: simplifyClauses(data.and)
    } : {}, {}, data.or.length ? {
      should: simplifyClauses(data.or)
    } : {}, {}, data.not.length ? {
      must_not: simplifyClauses(data.not)
    } : {}, {}, data.filter ? {
      filter: data.filter
    } : {}, {}, data.minimumShouldMatch != null && totalCount > 1 ? {
      minimum_should_match: data.minimumShouldMatch
    } : {})
  };
}

function buildBody(data) {
  var built = {};
  var aggregations = data.aggregations,
      filter = data.filter,
      query = data.query;

  function buildClauses(clauses) {
    return clauses.map(function (args) {
      return buildClause(data, args);
    }).filter(function (c) {
      return c;
    });
  }

  var builtFilter = buildBool({
    and: buildClauses(filter.and),
    or: buildClauses(filter.or),
    not: buildClauses(filter.not),
    minimumShouldMatch: filter.minimumShouldMatch
  });

  if (data.inChildContext) {
    if (builtFilter) {
      built.filter = builtFilter;
    }
  }

  var builtQuery = buildBool({
    and: buildClauses(query.and),
    or: buildClauses(query.or),
    not: buildClauses(query.not),
    filter: data.inChildContext ? undefined : builtFilter,
    minimumShouldMatch: query.minimumShouldMatch
  });

  if (builtQuery) {
    built.query = builtQuery;
  }

  if (data.from != null) {
    built.from = data.from;
  }

  if (data.size != null) {
    built.size = data.size;
  }

  if (data.sort.length) {
    built.sort = deduplicateSort(flatMap(data.sort, function (args) {
      return buildSort(args);
    }));
  }

  if (is.nonEmptyObject(aggregations)) {
    built.aggs = aggregations.reduce(function (acc, args) {
      return _extends({}, acc, {}, buildAggregation(args));
    }, {});
  }

  return _extends({}, built, {}, data.rawOption);
}

/** Returns a new Elasticsearch query builder instance */

function esBuilder() {
  return builderBuilder();
}

function defaultData() {
  return {
    aggregations: [],
    filter: {
      and: [],
      or: [],
      not: [],
      minimumShouldMatch: undefined
    },
    from: undefined,
    inChildContext: false,
    size: undefined,
    sort: [],
    query: {
      and: [],
      or: [],
      not: [],
      minimumShouldMatch: undefined
    },
    rawOption: {}
  };
}

function prepareSubBuilderArgs(args, replacement) {
  args.forEach(function (arg, i) {
    if (is.function_(arg)) {
      args[i] = function () {
        return arg(replacement());
      };
    }
  });
  return args;
}
/** Returns the entire ES query builder */


function builderBuilder(data) {
  if (data === void 0) {
    data = defaultData();
  }

  return _extends({
    from: function from(quantity) {
      return builderBuilder(_extends({}, data, {
        from: quantity
      }));
    },
    size: function size(quantity) {
      return builderBuilder(_extends({}, data, {
        size: quantity
      }));
    },
    rawOption: function rawOption(key, value) {
      var _extends2;

      return builderBuilder(_extends({}, data, {
        rawOption: _extends({}, data.rawOption, (_extends2 = {}, _extends2[key] = value, _extends2))
      }));
    },
    sort: function sort() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return builderBuilder(_extends({}, data, {
        sort: [].concat(data.sort, [args])
      }));
    }
  }, aggregationBuilder(data), {}, baseBuilder(data), {}, filterBuilder(data), {}, queryBuilder(data));
}
/** Returns builder functions for aggregations */


function aggregationBuilder(data) {
  var aggregations = data.aggregations;

  function bindSubBuilder(args) {
    return prepareSubBuilderArgs(args, function () {
      var next = _extends({}, defaultData(), {
        inChildContext: true
      });

      return _extends({}, aggregationBuilder(next), {}, baseBuilder(next), {}, filterBuilder(next), {}, queryBuilder(next));
    });
  }

  var aggregation = function aggregation() {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return builderBuilder(_extends({}, data, {
      aggregations: [].concat(aggregations, [bindSubBuilder(args)])
    }));
  };

  return {
    agg: aggregation,
    aggregation: aggregation
  };
}
/** Returns base builder function */


function baseBuilder(data) {
  return {
    build: function build() {
      return buildBody(data);
    }
  };
}
/** Returns builder functions for filters */


function filterBuilder(data) {
  var _filter = data.filter;

  function bindSubBuilder(args) {
    return prepareSubBuilderArgs(args, function () {
      var next = _extends({
        parent: 'filter'
      }, defaultData(), {
        inChildContext: true
      });

      return _extends({}, baseBuilder(next), {}, filterBuilder(next), {}, data.parent === 'query' ? queryBuilder(next) : {});
    });
  }

  return {
    filter: function filter() {
      for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      return builderBuilder(_extends({}, data, {
        filter: _extends({}, _filter, {
          and: [].concat(_filter.and, [bindSubBuilder(args)])
        })
      }));
    },
    andFilter: function andFilter() {
      for (var _len4 = arguments.length, args = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      return builderBuilder(_extends({}, data, {
        filter: _extends({}, _filter, {
          and: [].concat(_filter.and, [bindSubBuilder(args)])
        })
      }));
    },
    orFilter: function orFilter() {
      for (var _len5 = arguments.length, args = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        args[_key5] = arguments[_key5];
      }

      return builderBuilder(_extends({}, data, {
        filter: _extends({}, _filter, {
          or: [].concat(_filter.or, [bindSubBuilder(args)])
        })
      }));
    },
    notFilter: function notFilter() {
      for (var _len6 = arguments.length, args = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        args[_key6] = arguments[_key6];
      }

      return builderBuilder(_extends({}, data, {
        filter: _extends({}, _filter, {
          not: [].concat(_filter.not, [bindSubBuilder(args)])
        })
      }));
    },
    filterMinimumShouldMatch: function filterMinimumShouldMatch(minimumShouldMatch) {
      return builderBuilder(_extends({}, data, {
        filter: _extends({}, _filter, {
          minimumShouldMatch: minimumShouldMatch
        })
      }));
    }
  };
}
/** Returns builder functions for queries */


function queryBuilder(data) {
  var _query = data.query;

  function bindSubBuilder(args) {
    return prepareSubBuilderArgs(args, function () {
      var next = _extends({
        parent: 'query'
      }, defaultData(), {
        inChildContext: true
      });

      return _extends({}, baseBuilder(next), {}, filterBuilder(next), {}, queryBuilder(next));
    });
  }

  return {
    query: function query() {
      for (var _len7 = arguments.length, args = new Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        args[_key7] = arguments[_key7];
      }

      return builderBuilder(_extends({}, data, {
        query: _extends({}, _query, {
          and: [].concat(_query.and, [bindSubBuilder(args)])
        })
      }));
    },
    andQuery: function andQuery() {
      for (var _len8 = arguments.length, args = new Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
        args[_key8] = arguments[_key8];
      }

      return builderBuilder(_extends({}, data, {
        query: _extends({}, _query, {
          and: [].concat(_query.and, [bindSubBuilder(args)])
        })
      }));
    },
    orQuery: function orQuery() {
      for (var _len9 = arguments.length, args = new Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
        args[_key9] = arguments[_key9];
      }

      return builderBuilder(_extends({}, data, {
        query: _extends({}, _query, {
          or: [].concat(_query.or, [bindSubBuilder(args)])
        })
      }));
    },
    notQuery: function notQuery() {
      for (var _len10 = arguments.length, args = new Array(_len10), _key10 = 0; _key10 < _len10; _key10++) {
        args[_key10] = arguments[_key10];
      }

      return builderBuilder(_extends({}, data, {
        query: _extends({}, _query, {
          not: [].concat(_query.not, [bindSubBuilder(args)])
        })
      }));
    },
    queryMinimumShouldMatch: function queryMinimumShouldMatch(minimumShouldMatch) {
      return builderBuilder(_extends({}, data, {
        query: _extends({}, _query, {
          minimumShouldMatch: minimumShouldMatch
        })
      }));
    }
  };
}

export { esBuilder };
//# sourceMappingURL=elastic-ts.esm.js.map
